#!/usr/bin/env python3

import ivi
import cal
import winsound

###############################################################################
# Change these adreses as appropriate for your instruments
###############################################################################

# 
# DM3068
# DP831 and DP831A
#

dmm = ivi.rigol.rigolDM3068Agilent("TCPIP0::192.168.88.47::inst0::INSTR")
psu = ivi.rigol.rigolDP831A("TCPIP0::192.168.88.43::inst0::INSTR")


###############################################################################
# Settings
###############################################################################

# Current limit of your DMM's current range
#
# If this is less than 3.2A then you will need an alternative DMM for manual
# entry for currents over this limit.

manual_entry_over_current = 5

# Range of channels to calibrate
#
# Set to range(<min channel>, <max channel + 1>)

#calibrate_channels = range(1, 4)    # All Channels
calibrate_channels = range(1, 2)    # Channel 1 only ( 8V/5A )
#calibrate_channels = range(2, 3)    # Channel 2 only ( 30V/2A )
#calibrate_channels = range(3, 4)    # Channel 3 only (-30V/2A )
#calibrate_channels = range(2, 4)    # Channel 2&3 only

###############################################################################
# Callback function used to setup the DMM for either voltage or current
# function is either dc_volts or dc_current
###############################################################################

def setup_dmm(dmm, function):
    dmm.measurement_function = function
    dmm.auto_range = 'on'
    
 
    dmm.nplc = 1


###############################################################################
# WARNING: CALIBRATION VALUES WILL BE CHANGED IF THIS IS SET TO True
###############################################################################

# Update the calibration values for the DMM when True, or just run through
# calibration without updating the values if False.
#
# Warning: When False, the calibration values of the DP832 will still be cleared
# so you will have to restart the PSU to get them back. The calibration for any
# channels affected will be (probably wildly) off until you either restart or
# set this to True.

update_calibration = False

###############################################################################
# You should not need to change anything below here. Probably.
###############################################################################

psu.utility.reset()
dmm.utility.reset()

try:
    calibrator = cal.DP831ACal(psu, dmm)
    calibrator.dmm_setup_callback = setup_dmm
    calibrator.manual_current_limit = manual_entry_over_current
    calibrator.calibrate(calibrate_channels, update_calibration)
except (Exception, KeyboardInterrupt) as e:
    print()
    
    # Resetting will also turn off all outputs
    psu.utility.reset()

    winsound.Beep (440,100)
    print("ERROR: %s" % str(e))
    print("Calibration failed. Your DP831A may need to be restarted to regain calibration values.")
